import logging

from slacker import Slacker

tpl = """
=====================
{summary}
---------------------
{details}
=====================
"""

root_logger = logging.getLogger('root')
root_stream_handler = logging.StreamHandler()

try:
    from celery.utils.log import get_task_logger
    root_logger = get_task_logger('root')
except ImportError:
    pass

root_logger.addHandler(root_stream_handler)

try:
    from celery.signals import after_setup_task_logger, after_setup_logger

    @after_setup_logger.connect
    def after_setup_aklogger(logger, *args, **kwargs):
        root_logger.removeHandler(root_stream_handler)

    @after_setup_task_logger.connect
    def after_setup_task_aklogger(logger, *args, **kwargs):
        root_logger.removeHandler(root_stream_handler)
except ImportError:
    pass


def get_logger(name):
    return root_logger.getChild(name)


CRITICAL = logging.CRITICAL
ERROR = logging.ERROR
WARNING = logging.WARNING
INFO = logging.INFO
DEBUG = logging.DEBUG
NOTSET = logging.NOTSET

_levelToName = {
    CRITICAL: 'CRITICAL',
    ERROR: 'ERROR',
    WARNING: 'WARNING',
    INFO: 'INFO',
    DEBUG: 'DEBUG',
    NOTSET: 'NOTSET',
}


class AKLogger(object):

    def __init__(self, name, parent=None):
        self.set_name(name)
        self.parent = parent
        self.logger = get_logger(self.get_name())
        self.slkr = None
        self.slack_token = None
        self.slack_level = None
        self.setLevel(NOTSET)

    def set_name(self, name):
        self.name = name

    def get_name(self):
        if self.parent is not None:
            return self.parent.get_name() + ':' + self.name
        return self.name

    def getLogger(self, name):
        return AKLogger(name, parent=self)

    def setLevel(self, level):
        self.level = logging._checkLevel(level)
        self.logger.setLevel(self.level)

    def getLevel(self):
        return self.level

    def enable_slack(self, token):
        self.slkr = Slacker(token)
        self.slkr.api.test()
        self.slack_token = token

    def disable_slack(self):
        self.slkr = None
        self.slack_token = None

    def get_slack_level(self):
        return self.slack_level or self.level

    def set_slack_level(self, level):
        self.slack_level = logging._checkLevel(level)

    def log_to_file(self, file_name):
        self.filename = file_name
        handler = logging.FileHandler(self.filename)
        self.logger.addHandler(handler)

    def debug(self, summary, details=None, channel='#error',
              *args, **kwargs):
        self._log(DEBUG, self.get_name(), summary, details, channel, *args,
                  **kwargs)

    def info(self, summary, details=None, channel='#error', *args,
             **kwargs):
        self._log(INFO, self.get_name(), summary, details, channel, *args,
                  **kwargs)

    def warning(self, summary, details=None, channel='#error', *args,
                **kwargs):
        self._log(WARNING, self.get_name(), summary, details, channel, *args,
                  **kwargs)

    def error(self, summary, details=None, channel='#error', *args,
              **kwargs):
        self._log(ERROR, self.get_name(), summary, details, channel, *args,
                  **kwargs)

    def _log(self, level, name, summary=None, details=None, channel='#error',
             *args, **kwargs):

        if self.parent:
            self.parent._log(level, name, summary, details, channel, *args,
                             **kwargs)

        if level < self.getLevel() or self.getLevel() == NOTSET:
            return

        msg = summary
        if details:
            msg = tpl.format(
                summary=summary,
                details=details
            )
        method_to_call = getattr(self.logger, _levelToName.get(level).lower())
        method_to_call(msg, *args, **kwargs)
        self.slack_alert(summary, details, channel, level)

    def slack_alert(self, summary, details, channel, level,
                    color='good'):
        if not self.slkr:
            return
        if level < self.get_slack_level():
            return
        try:
            self.slkr.chat.post_message(
                channel=channel,
                text=summary,
                username=self.name,
                icon_emoji=':squirrel:',
                attachments=[
                    {
                        'fallback': summary,
                        'text': details,
                        'color': color,
                    }
                ] if details else None,
            )
        except Exception as e:
            self.logger.error('Slack failed: {}'.format(e))


logger = AKLogger('root')


def getLogger(name=None):
    if name is None:
        return logger
    return logger.getLogger(name)



import logging
from aklogger import logger


class DjangoHandler(logging.Handler):

    def __init__(self, name, slack_token=None,
                 slack_level=logging.ERROR, filename=None):
        super().__init__()
        logger.set_name(name)
        if slack_token:
            logger.enable_slack(slack_token)
        logger.set_slack_level(slack_level)
        if filename:
            logger.log_to_file(filename)

    def setLevel(self, level):
        logger.setLevel(level)
        super(DjangoHandler, self).setLevel(level)

    def emit(self, record):
        method_to_call = getattr(logger, record.levelname.lower())
        method_to_call(record.getMessage())



import os

from setuptools import setup, find_packages

__VERSION__ = '0.1.0'


def readme(*paths):
    with open(os.path.join(*paths), 'r') as f:
        return f.read()


setup(
    name="aklogger",
    version=__VERSION__,
    author="appknox",
    author_email="engineering@appknox.com",
    description="A generic logging package for python projects",
    long_description=readme('README.md'),
    url="https://github.com/appknox/aklogger",
    packages=find_packages(),
    license='MIT',
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'slacker>=0.7.3,<=0.9.65',
        'six==1.11.0'
    ],
    extras_require={
        'dev': [
            'bumpversion==0.5.3',
            'twine==1.12.1',
            'flake8==3.7.7'
        ],
        'test': [
            'codecov==2.0.15',
            'coverage==4.5.2',
            'pytest-cov==2.6.1'
        ],
    },
    keywords='appknox aklogger',
    entry_points='',
)



import os
from aklogger import logger, tpl
import slacker

logger.set_name('mycroft')
try:
    logger.enable_slack('invalid_token')
except slacker.Error:
    pass
logger.setLevel('DEBUG')
logger.set_slack_level('WARNING')
logger.log_to_file('file.log')


def test_debug():
    title = 'This is a debug message'
    desc = 'This is the description of the debug message'
    logger.debug(title, desc, '#error')
    msg = tpl.format(
        summary=title,
        details=desc
    )
    with open('file.log', 'r+') as f:
        content = f.read()
        f.truncate(0)
    assert len(msg) + 1 == len(content)


def test_info():
    title = 'This is a info message'
    desc = 'This is the description of the info message'
    logger.info(title, desc, '#error')
    msg = tpl.format(
        summary=title,
        details=desc
    )
    with open('file.log', 'r+') as f:
        content = f.read()
        f.truncate(0)
    assert len(msg) + 1 == len(content)


def test_warning():
    title = 'This is a warning message'
    desc = 'This is the description of the warning message'
    logger.warning(title, desc, '#error')
    msg = tpl.format(
        summary=title,
        details=desc
    )
    # extra_slack_log_for_invalid_token_or_channel
    extra_log = 'Slack failed: invalid_auth'
    msg2 = msg + '\n' + extra_log
    with open('file.log', 'r+') as f:
        content = f.read()
        f.truncate(0)
    assert len(msg2) + 1 == len(content)


def test_error():
    title = 'This is a error message'
    desc = None
    logger.error(title, desc, '#error')
    # extra_slack_log_for_invalid_token_or_channel
    extra_log = 'Slack failed: invalid_auth'
    msg2 = title + '\n' + extra_log
    with open('file.log', 'r+') as f:
        content = f.read()
        f.truncate(0)
    assert len(msg2) + 1 == len(content)
    os.remove('file.log')
